import unittest
import os
import sys
from psycopg2 import connect, extensions, sql
from datetime import datetime

from p2.store.client import Client
from p2.tools.prepare_db import setUpDB, tearDownDB


class TestClient(unittest.TestCase):

    def setUp(self):
        self.p2_store_url = os.environ.get('P2_STORE_URL')

    def test_get_version(self):
        client = Client(self.p2_store_url, timeout=60)
        version = client.get_version()
        self.assertIsNotNone(version)

    def test_get_campaigns(self):
        client = Client(self.p2_store_url)
        campaigns = client.get_campaigns()
        self.assertEqual(len(campaigns), 3)

    def test_get_campaign(self):
        client = Client(self.p2_store_url)
        campaigns = client.get_campaigns()
        campaign = client.get_campaign_of_id(campaigns[0]['id'])
        self.assertEqual(campaign['id'], campaigns[0]['id'])

    def test_get_experiment(self):
        client = Client(self.p2_store_url)
        data = client.get_experiment('exp_1')
        self.assertEqual(data, {"title":"experiment 1"})

    def test_get_experiments(self):
        client = Client(self.p2_store_url)
        records = client.get_experiments(filename='exp_1')
        self.assertEqual(len(records), 1)
        self.assertEqual(records[0]['filename'], 'exp_1')

    def test_get_experiments_of_campaign(self):
        client = Client(self.p2_store_url)
        campaigns = client.get_campaigns()
        records = client.get_experiments(
                campaigns[0]['experiments']
                )
        self.assertEqual(len(records), 5)
        self.assertEqual(records[0]['filename'], 'exp_1')

    def test_get_experiments_as_dict(self):
        client = Client(self.p2_store_url)
        records = client.get_experiments_as_dict()
        self.assertEqual(len(records.keys()), 7)

    def test_add_delete_experiment(self):
        client = Client(self.p2_store_url)
        exp = client.add_experiment(
                'exp_6', {'k1':'v1'}
        )
        self.assertIsNotNone(exp)
        self.assertEqual(exp['filename'], 'exp_6')

        exp = client.delete_experiment('exp_6')
        self.assertEqual(exp, 'exp_6')

    def test_create_session(self):
        client = Client(self.p2_store_url)
        time = datetime.now()
        campaigns = client.get_campaigns()
        campaign_id = campaigns[0]['id']
        id = client.create_session('new-session-with-target', {
            'filepath': 'exp_1',
            'campaign_id': campaign_id,
            'schedule': '0 0 * * *',
            'start_time': str(time),
            'end_time': str(time),
            'status': 'completed',
            'profile': {
                'target': {
                    'context': 'minikube',
                    'id': 1,
                    'kind': 'namespace',
                    'last_updated': '2021-02-11T12:28:41.298053',
                    'name': 'reference-app-int-nft',
                    'parent_id': None,
                    'profile': {'area': 'core',
                                'name': 'reference-app-int-nft',
                                'team': 'core-community'},
                    'status': 'active'
                }
            }
        })
        self.assertEqual(id, 'new-session-with-target')

    def test_get_sessions(self):
        client = Client(self.p2_store_url)
        sessions = client.get_sessions(exp_id='1')
        self.assertEqual(len(sessions), 1)
        self.assertEqual(sessions[0]['exp_id'], '1')

    def test_update_session(self):
        client = Client(self.p2_store_url)
        id = client.update_session('1', {'status': 'scheduled'})
        self.assertEqual(id, '1')
        sessions = client.get_sessions(exp_id='1')
        self.assertEqual(sessions[0]['status'], 'scheduled')

    def test_delete_session(self):
        client = Client(self.p2_store_url)
        id = client.delete_session('1')
        self.assertEqual(id, '1')
        sessions = client.get_sessions(exp_id='1')
        self.assertEqual(sessions[0]['status'], 'deleted')

    def test_update_log(self):
        client = Client(self.p2_store_url)
        id = client.update_log(
                '1', {
                    'log_type': 1,
                    'log_text': 'log text'
                })
        self.assertEqual(id, '1')

    def test_get_exp_log(self):
        client = Client(self.p2_store_url)
        log = client.get_exp_log('1', 1)
        self.assertEqual(log[0]['log_type'], 1)
        self.assertEqual(log[0]['log_text'], 'logtexttype1')

    def test_execute_command(self):
        client = Client(self.p2_store_url)
        session = client.execute_command(
                cmd="select * from exp_session where exp_id = %s",
                attrs=['1']
        )
        self.assertEqual(session[0]['status'], 'deleted')

    def test_get_config(self):
        client = Client(self.p2_store_url)
        config = client.get_config('p2.scheduler')
        self.assertEqual(config['spare_days_per_month'], 1)

    def test_get_config_with_tag(self):
        client = Client(self.p2_store_url)
        config = client.get_config('p2.scheduler', tag='minikube')
        self.assertEqual(config['spare_days_per_month'], 1)

    def test_get_resources(self):
        client = Client(self.p2_store_url)
        resources = client.get_resources(**{
            'name':'org 1',
            'kind':'org'
        })
        self.assertIsNotNone(resources)
        self.assertTrue(isinstance(resources, dict))
        self.assertEqual(resources['children'][0]['name'], 'region 1')

    def test_get_resources_faster(self):
        client = Client(self.p2_store_url)
        resources = client.get_resources_faster(**{
            'name':'org 1',
            'kind':'org'
        })
        self.assertIsNotNone(resources)
        self.assertTrue(isinstance(resources, dict))
        self.assertEqual(resources['children'][0]['name'], 'region 1')

    def test_select(self):
        client = Client(self.p2_store_url)
        records = client.select(
                table='exp_session',
                fetch=['exp_id', 'filepath'],
                sort='exp_id',
                limit=6,
                offset=0,
                filter={'context':'minikube'}
            )
        self.assertEqual(len(records), 6)
        self.assertEqual(len(records[0].keys()), 2)
        self.assertEqual(records[0]['exp_id'], '1')

    def test_select_like(self):
        client = Client(self.p2_store_url)
        records = client.select(
                table='exp_session',
                fetch=['exp_id', 'filepath'],
                sort='exp_id',
                limit=6,
                offset=0,
                filter={'context|like':'%mini%'}
            )
        self.assertEqual(len(records), 6)
        self.assertEqual(len(records[0].keys()), 2)
        self.assertEqual(records[0]['exp_id'], '1')

    def test_select_not_like(self):
        client = Client(self.p2_store_url)
        records = client.select(
                table='exp_session',
                fetch=['exp_id', 'filepath'],
                sort='exp_id',
                limit=6,
                offset=0,
                filter={
                    'context|like':'%mini%',
                    'filepath|not like': ['%to_skip_1%', '%to_skip_2%']
                }
            )
        self.assertEqual(len(records), 6)
        self.assertEqual(len(records[0].keys()), 2)
        self.assertEqual(records[0]['exp_id'], '1')

    def test_select_no_limit_offset(self):
        client = Client(self.p2_store_url)
        records = client.select(
                table='exp_session',
                fetch=['exp_id', 'filepath'],
                sort='exp_id',
                filter={'context':'minikube'}
            )
        self.assertEqual(len(records), 9)
        self.assertEqual(len(records[0].keys()), 2)
        self.assertEqual(records[0]['exp_id'], '1')


    def test_select_only_table_filter(self):
        client = Client(self.p2_store_url)
        records = client.select(
                table='exp_session',
                filter={'context':'minikube'}
            )
        self.assertEqual(len(records), 9)
        self.assertEqual(len(records[0].keys()), 17)


    def test_select_table_only(self):
        client = Client(self.p2_store_url)
        records = client.select(table='exp_session')
        self.assertEqual(len(records), 13)
        self.assertEqual(len(records[0].keys()), 17)


    def test_add(self):
        client = Client(self.p2_store_url)
        resp = client.add(
                table='reports',
                record={
                    'params': '"params"',
                    'tag': 'minikube',
                    'report_type': 'access_review_report',
                    'title': 'Access Review Report on context minikube'
                    }
            )
        self.assertEqual(resp, 1)


    def test_get_namespace_annotations(self):
        client = Client(self.p2_store_url)
        a = client.get_namespace_annotations('ns1')
        self.assertEqual(a, {"team": "The best"})


    def test_add_returning_key(self):
        client = Client(self.p2_store_url)
        id = client.add_returning_key(
                table='reports',
                key='id',
                record={
                    'params': '"params"',
                    'tag': 'minikube',
                    'report_type': 'access_review_report_2',
                    'title': 'Access Review Report on context minikube'
                    }
            )
        self.assertGreater(id, 1)
        report = client.select(
                table='reports',
                sort='creation_time desc',
                filter={'id': id}
                )
        self.assertIsNotNone(report)
        self.assertGreater(len(report),0)
        self.assertEqual(report[0]['report_type'], 'access_review_report_2')

    def test_update_on_conflict(self):
        client = Client(self.p2_store_url)
        ret = client.update_on_conflict(
                table='resources',
                conflicting_cols=['id'],
                record={
                    'id': 8,
                    'layer': 'k8',
                    'kind': 'namespace',
                    'name': 'ns1',
                    'status': 'active',
                    'last_updated': datetime.now().isoformat()
                    }
            )
        self.assertIsNotNone(ret)
        resource = client.select(
                table='resources',
                sort='last_updated desc',
                limit=1,
                filter={
                    'layer': 'k8',
                    'kind': 'namespace',
                    'name': 'ns1'
                    }
                )
        self.assertIsNotNone(resource)
        self.assertGreater(len(resource),0)
        self.assertEqual(resource[0]['status'], 'active')

        res = client.update_on_conflict(
                table='resources',
                conflicting_cols=['id'],
                record={
                    'id': 8,
                    'layer': 'k8',
                    'kind': 'namespace',
                    'name': 'ns1',
                    'status': 'partially offline',
                    'last_updated': datetime.now().isoformat()
                    }
            )
        resource = client.select(
                table='resources',
                sort='last_updated desc',
                limit=1,
                filter={
                    'layer': 'k8',
                    'kind': 'namespace',
                    'name': 'ns1'
                    }
                )
        self.assertIsNotNone(resource)
        self.assertGreater(len(resource),0)
        self.assertEqual(resource[0]['status'], 'partially offline')

    def test_delete(self):
        client = Client(self.p2_store_url)
        res = client.delete(
            table='reports',
            key='title',
            value='AccessReviewReport'
        )
        self.assertEqual(res, 1)

    def test_update(self):
        client = Client(self.p2_store_url)
        res = client.update(
                table='resources',
                record={
                    'status': 'kaput',
                    'last_updated': datetime.now().isoformat()
                },
                where={'name': 'ns1'}
            )
        self.assertEqual(res, 1)
        resource = client.select(
                table='resources',
                sort='last_updated desc',
                limit=1,
                filter={
                    'name': 'ns1'
                    }
                )
        self.assertIsNotNone(resource)
        self.assertGreater(len(resource),0)
        self.assertEqual(resource[0]['status'], 'kaput')
